@@ -40,24 +40,41 @@
 #include "app-profiles.h"
 #include "msg.h"
 
+/*
+ * Define a wrapper around json_object_foreach() for compatibility with older
+ * versions of libjansson.
+ */
+#if JANSSON_VERSION_HEX < 0x020200
+# error "nvidia-settings requires jansson version 2.2 or later.  Please update"
+# error "your version of jansson, or set NV_USE_BUNDLED_LIBJANSSON=1 to build"
+# error "with the version of jansson included with the nvidia-settings source"
+# error "code."
+#elif JANSSON_VERSION_HEX < 0x020300
+# define NV_JSON_OBJECT_FOREACH(object, key, value) \
+    for(key = json_object_iter_key(json_object_iter(object)); \
+        key && (value = json_object_iter_value(json_object_iter_at(object, key))); \
+        key = json_object_iter_key(json_object_iter_next(object, json_object_iter_at(object, key))))
+#else
+# define NV_JSON_OBJECT_FOREACH(object, key, value) json_object_foreach(object, key, value)
+#endif
+
 static char *slurp(FILE *fp)
 {
     int eof = FALSE;
     char *text = strdup("");
     char *new_text;
-    char *line = NULL;
 
     while (text && !eof) {
-        line = fget_next_line(fp, &eof);
-        if (!eof) {
+        char *line = fget_next_line(fp, &eof);
+
+        if (line && *line != '\0' && *line != '\n') {
             new_text = nvstrcat(text, "\n", line, NULL);
             free(text);
             text = new_text;
         }
+        free(line);
     }
 
-    free(line);
-
     return text;
 }
 
@@ -74,7 +91,7 @@ static void splice_string(char **s, size_t b, size_t e, const char *replace)
 
 #define HEX_DIGITS "0123456789abcdefABCDEF"
 
-char *nv_app_profile_cfg_file_syntax_to_json(const char *orig_s)
+char *nv_app_profile_file_syntax_to_json(const char *orig_s)
 {
     char *s = strdup(orig_s);
 
@@ -189,16 +206,6 @@ static int open_and_stat(const char *filename, const char *perms, FILE **fp, str
     return ret;
 }
 
-static char *nv_dirname(const char *path)
-{
-    char *last_slash = strrchr(path, '/');
-    if (last_slash) {
-        return nvstrndup(path, last_slash - path);
-    } else {
-        return nvstrdup(".");
-    }
-}
-
 static json_t *app_profile_config_insert_file_object(AppProfileConfig *config, json_t *new_file)
 {
     json_t *json_filename, *json_new_filename;
@@ -427,6 +434,119 @@ static json_t *json_settings_parse(json_t *old_settings, const char *filename)
     return new_settings;
 }
 
+/*
+ * Load app profile key documentation from file.
+ */
+json_t *nv_app_profile_key_documentation_load(const char *key_docs_file)
+{
+    json_t *key_docs = NULL;
+    int ret, i;
+    struct stat stat_buf;
+    FILE *fp = NULL;
+
+    char *orig_text = NULL;
+    char *json_text = NULL;
+    json_t *orig_file = NULL;
+    json_t *orig_json_keys = NULL;
+    json_error_t error;
+
+    if (!key_docs_file) {
+        goto done;
+    }
+
+    ret = open_and_stat(key_docs_file, "r", &fp, &stat_buf);
+    if (ret < 0) {
+        goto done;
+    }
+
+    orig_text = slurp(fp);
+
+    if (!orig_text) {
+        nv_error_msg("Could not read from file %s", key_docs_file);
+        goto done;
+    }
+
+    // Convert the file contents to JSON
+    json_text = nv_app_profile_file_syntax_to_json(orig_text);
+
+    if (!json_text) {
+        nv_error_msg("App profile parse error in %s: "
+                     "text is not valid app profile key "
+                     "documentation syntax", key_docs_file);
+        goto done;
+    }
+
+    // Parse the resulting JSON
+    orig_file = json_loads(json_text, 0, &error);
+
+    if (!orig_file) {
+        nv_error_msg("App profile parse error in %s: %s on %s, line %d\n",
+                       key_docs_file, error.text, error.source, error.line);
+        goto done;
+    }
+
+    // Process the array of key objects within the top level object
+    orig_json_keys = json_object_get(orig_file, "registry_keys");
+
+    if (orig_json_keys) {
+        int size = json_array_size(orig_json_keys);
+
+        key_docs = json_array();
+
+        for (i = 0; i < size; i++) {
+            json_t *json_name, *json_description, *json_type;
+            json_t *json_key_object = json_array_get(orig_json_keys, i);
+
+            if (!json_is_object(json_key_object)) {
+                nv_error_msg("App profile parse error in %s: "
+                             "Object expected in 'registry_keys' array "
+                             "at position %d",
+                             key_docs_file, i);
+                continue;
+            }
+
+            json_name        = json_object_get(json_key_object, "key");
+            json_description = json_object_get(json_key_object, "description");
+            json_type        = json_object_get(json_key_object, "type");
+
+            /*
+             * Any invalid and non-string type for any fields per key will
+             * cause the key's data to not be added.
+             */
+            if (json_is_string(json_name) &&
+                json_is_string(json_description) &&
+                json_is_string(json_type)) {
+
+                json_t *new_json_key_object = json_object();
+
+                json_object_set(new_json_key_object, "key", json_name);
+                json_object_set(new_json_key_object, "description",
+                                json_description);
+                json_object_set(new_json_key_object, "type", json_type);
+
+                json_array_append_new(key_docs, new_json_key_object);
+            }
+        }
+    }
+
+
+done:
+    if (json_array_size(key_docs) == 0) {
+        json_decref(key_docs);
+        key_docs = NULL;
+    }
+
+    free(orig_text);
+    free(json_text);
+    json_decref(orig_file);
+
+    if (fp) {
+        fclose(fp);
+    }
+
+    return key_docs;
+}
+
 /*
  * Load app profile settings from an already-open file. This operation is
  * atomic: either all of the settings from the file are added to the
@@ -462,7 +582,7 @@ static void app_profile_config_load_file(AppProfileConfig *config,
     }
 
     // Convert the file contents to JSON
-    json_text = nv_app_profile_cfg_file_syntax_to_json(orig_text);
+    json_text = nv_app_profile_file_syntax_to_json(orig_text);
 
     if (!json_text) {
         nv_error_msg("App profile parse error in %s: text is not valid app profile configuration syntax", filename);
@@ -674,7 +794,7 @@ static void app_profile_config_load_file(AppProfileConfig *config,
     {
         const char *key;
         json_t *value;
-        json_object_foreach(new_json_profiles, key, value) {
+        NV_JSON_OBJECT_FOREACH(new_json_profiles, key, value) {
             json_object_set_new(config->profile_locations, key, json_string(filename));
         }
     }
@@ -769,6 +889,9 @@ static json_t *app_profile_config_load_global_options(const char *global_config_
 
     ret = open_and_stat(global_config_file, "r", &fp, &stat_buf);
     if ((ret < 0) || !S_ISREG(stat_buf.st_mode)) {
+        if (ret >= 0) {
+            fclose(fp);
+        }
         return options;
     }
 
@@ -866,49 +989,30 @@ static int file_in_search_path(AppProfileConfig *config, const char *filename)
     return FALSE;
 }
 
+// Print an error message and optionally capture the error string for later use
+// Note: this assumes fmt is a string literal!
+#define LOG_ERROR(error_str, fmt, ...) do {                   \
+    if (error_str) {                                          \
+        nv_append_sprintf(error_str, fmt "\n", __VA_ARGS__);  \
+    }                                                         \
+    nv_error_msg(fmt, __VA_ARGS__);                           \
+} while (0)
+
 /*
- * Creates parent directories as needed, similarly to "mkdir -p"
+ * Create parent directories as needed and handle error messages
  */
-static int nv_mkdirp(const char *dirname)
+static int nv_mkdirp(const char *dirname, char **error_str)
 {
-    int ret = 0;
-    char *parent_name;
-    const char *cur, *next;
-    struct stat stat_buf;
-    cur = dirname;
+    char *tmp_error_str = NULL;
+    int success;
 
-    while (*cur && (next = strchr(cur + 1, '/'))) {
-        parent_name = nvstrndup(dirname, next - dirname);
-        ret = mkdir(parent_name, 0777);
-        if ((ret < 0) && (errno != EEXIST)) {
-            nv_error_msg("Could not create parent directory \"%s\" for full path \"%s\" (%s)",
-                         parent_name, dirname, strerror(errno));
-            free(parent_name);
-            return ret;
-        }
-        cur = next;
-        free(parent_name);
+    success = nv_mkdir_recursive(dirname, 0777, &tmp_error_str, NULL);
+    if (tmp_error_str) {
+        LOG_ERROR(error_str, "%s", tmp_error_str);
+        free(tmp_error_str);
     }
 
-    ret = mkdir(dirname, 0777);
-    if (ret < 0) {
-        if (errno != EEXIST) {
-            nv_error_msg("Could not create directory \"%s\" (%s)",
-                         dirname, strerror(errno));
-        } else {
-            ret = stat(dirname, &stat_buf);
-            if (ret == 0) {
-                if (!S_ISDIR(stat_buf.st_mode)) {
-                    nv_error_msg("Could not create directory \"%s\" (file "
-                                 "exists, but not as a directory)",
-                                 dirname);
-                    ret = -1;
-                }
-            }
-        }
-    }
-
-    return ret;
+    return success ? 0 : -1;
 }
 
 char *nv_app_profile_config_get_backup_filename(AppProfileConfig *config, const char *filename)
@@ -941,16 +1045,17 @@ char *nv_app_profile_config_get_backup_filename(AppProfileConfig *config, const
 }
 
 static int app_profile_config_backup_file(AppProfileConfig *config,
-                                          const char *filename)
+                                          const char *filename,
+                                          char **error_str)
 {
     int ret;
     char *backup_name = nv_app_profile_config_get_backup_filename(config, filename);
     char *backup_dirname = nv_dirname(backup_name);
 
-    ret = nv_mkdirp(backup_dirname);
+    ret = nv_mkdirp(backup_dirname, error_str);
     if (ret < 0) {
-        nv_error_msg("Could not create backup directory \"%s\" (%s)",
-                     backup_name, strerror(errno));
+        LOG_ERROR(error_str, "Could not create backup directory \"%s\" (%s)",
+                  backup_name, strerror(errno));
         goto done;
     }
 
@@ -960,8 +1065,8 @@ static int app_profile_config_backup_file(AppProfileConfig *config,
             // Clear the error; the file does not exist
             ret = 0;
         } else {
-            nv_error_msg("Could not rename file \"%s\" to \"%s\" for backup (%s)",
-                         filename, backup_name, strerror(errno));
+            LOG_ERROR(error_str, "Could not rename file \"%s\" to \"%s\" for backup (%s)",
+                      filename, backup_name, strerror(errno));
         }
     }
 
@@ -973,10 +1078,12 @@ done:
     return ret;
 }
 
+
 static int app_profile_config_save_updates_to_file(AppProfileConfig *config,
                                                    const char *filename,
                                                    const char *update_text,
-                                                   int backup)
+                                                   int backup,
+                                                   char **error_str)
 {
     int file_is_new = FALSE;
     struct stat stat_buf;
@@ -987,7 +1094,8 @@ static int app_profile_config_save_updates_to_file(AppProfileConfig *config,
     ret = stat(filename, &stat_buf);
 
     if ((ret < 0) && (errno != ENOENT)) {
-        nv_error_msg("Could not stat file \"%s\" (%s)", filename, strerror(errno));
+        LOG_ERROR(error_str, "Could not stat file \"%s\" (%s)",
+                  filename, strerror(errno));
         goto done;
     } else if ((ret < 0) && (errno == ENOENT)) {
         file_is_new = TRUE;
@@ -998,11 +1106,12 @@ static int app_profile_config_save_updates_to_file(AppProfileConfig *config,
             // This file is in a directory in the search path
             ret = stat(dirname, &stat_buf);
             if ((ret < 0) && (errno != ENOENT)) {
-                nv_error_msg("Could not stat file \"%s\" (%s)", dirname, strerror(errno));
+                LOG_ERROR(error_str, "Could not stat file \"%s\" (%s)",
+                          dirname, strerror(errno));
                 goto done;
             } else if ((ret < 0) && errno == ENOENT) {
                 // Attempt to create the directory in the search path
-                ret = nv_mkdirp(dirname);
+                ret = nv_mkdirp(dirname, error_str);
                 if (ret < 0) {
                     goto done;
                 }
@@ -1010,24 +1119,27 @@ static int app_profile_config_save_updates_to_file(AppProfileConfig *config,
                 // If the search path entry is currently a regular file,
                 // unlink it and create a directory instead
                 if (backup) {
-                    ret = app_profile_config_backup_file(config, dirname);
+                    ret = app_profile_config_backup_file(config, dirname,
+                                                         error_str);
                     if (ret < 0) {
                         goto done;
                     }
                 }
                 ret = unlink(dirname);
                 if (ret < 0) {
-                    nv_error_msg("Could not remove the file \"%s\" (%s)", dirname, strerror(errno));
+                    LOG_ERROR(error_str,
+                              "Could not remove the file \"%s\" (%s)",
+                              dirname, strerror(errno));
                     goto done;
                 }
-                ret = nv_mkdirp(dirname);
+                ret = nv_mkdirp(dirname, error_str);
                 if (ret < 0) {
                     goto done;
                 }
             }
         } else {
             // Attempt to create parent directories for this file
-            ret = nv_mkdirp(dirname);
+            ret = nv_mkdirp(dirname, error_str);
             if (ret < 0) {
                 goto done;
             }
@@ -1037,18 +1149,23 @@ static int app_profile_config_save_updates_to_file(AppProfileConfig *config,
         // but that seems a little dangerous. Instead, complain and bail out
         // here.
         ret = -1;
-        nv_error_msg("Refusing to write to file \"%s\" since it is not a regular file", filename);
+        LOG_ERROR(error_str,
+                  "Refusing to write to file \"%s\" "
+                  "since it is not a regular file", filename);
+        goto done;
     }
 
     if (!file_is_new && backup) {
-        ret = app_profile_config_backup_file(config, filename);
+        ret = app_profile_config_backup_file(config, filename,
+                                             error_str);
         if (ret < 0) {
             goto done;
         }
     }
     ret = open_and_stat(filename, "w", &fp, &stat_buf);
     if (ret < 0) {
-        nv_error_msg("Could not write to the file \"%s\" (%s)", filename, strerror(errno));
+        LOG_ERROR(error_str, "Could not write to the file \"%s\" (%s)",
+                  filename, strerror(errno));
         goto done;
     }
     nv_info_msg("", "Writing to configuration file \"%s\"\n", filename);
@@ -1060,24 +1177,44 @@ done:
     return ret;
 }
 
-int nv_app_profile_config_save_updates(AppProfileConfig *config, json_t *updates, int backup)
+int nv_app_profile_config_save_updates(AppProfileConfig *config,
+                                       json_t *updates,
+                                       int backup,
+                                       char **error_str)
 {
     json_t *update;
     const char *filename;
     const char *update_text;
     size_t i, size;
     int ret = 0;
+    int all_ret = 0;
 
-    for (i = 0, size = json_array_size(updates); (ret == 0) && (i < size); i++) {
+    if (error_str) {
+        *error_str = NULL;
+    }
+
+    for (i = 0, size = json_array_size(updates); i < size; i++) {
         update = json_array_get(updates, i);
         filename = json_string_value(json_object_get(update, "filename"));
         update_text = json_string_value(json_object_get(update, "text"));
-        ret = app_profile_config_save_updates_to_file(config, filename, update_text, backup);
+        ret = app_profile_config_save_updates_to_file(config,
+                                                      filename,
+                                                      update_text,
+                                                      backup,
+                                                      error_str);
+        if (ret < 0) {
+            all_ret = -1;
+        }
     }
 
-    assert(ret <= 0);
+    assert(all_ret <= 0);
 
-    return ret;
+    // This asserts an error string is set iff we are returning an error
+    assert(!error_str ||
+           (!(*error_str) && (all_ret == 0)) ||
+           ((*error_str) && (all_ret < 0)));
+
+    return all_ret;
 }
 
 AppProfileConfig *nv_app_profile_config_dup(AppProfileConfig *config)
@@ -1255,7 +1392,7 @@ static char *config_to_cfg_file_syntax(json_t *old_rules, json_t *old_profiles)
     }
 
     if (old_profiles) {
-        json_object_foreach(old_profiles, profile_name, old_profile) {
+        NV_JSON_OBJECT_FOREACH(old_profiles, profile_name, old_profile) {
             new_profile = app_profile_config_profile_output(profile_name, old_profile);
             json_array_append_new(profiles_array, new_profile);
         }
@@ -1330,7 +1467,7 @@ json_t *nv_app_profile_config_validate(AppProfileConfig *new_config,
     add_files_from_config(old_config, all_files, changed_files);
 
     // For each file in the set, determine if it needs to be updated
-    json_object_foreach(all_files, filename, unused) {
+    NV_JSON_OBJECT_FOREACH(all_files, filename, unused) {
         app_profile_config_get_per_file_config(new_config, filename, &new_file, &new_rules, &new_profiles);
         app_profile_config_get_per_file_config(old_config, filename, &old_file, &old_rules, &old_profiles);
 
@@ -1341,7 +1478,7 @@ json_t *nv_app_profile_config_validate(AppProfileConfig *new_config,
     }
 
     // For each file that changed, generate an update record with the new JSON
-    json_object_foreach(changed_files, filename, unused) {
+    NV_JSON_OBJECT_FOREACH(changed_files, filename, unused) {
         update = json_object();
 
         json_object_set_new(update, "filename", json_string(filename));
